import importlib


CORE_MODULE_NAME = 'cores'


def get_iscore_class_str(cls):
    path, _ = cls.__module__.rsplit('.{}'.format(CORE_MODULE_NAME), 1)
    core = cls.__name__
    return '{path}.{core}'.format(
        path=path,
        core=core,
    )


def get_iscore_class(class_str):
    app_label, core = class_str.rsplit('.', 1)
    core_module_path = '.'.join((app_label, CORE_MODULE_NAME))

    module = importlib.import_module(core_module_path)

    return getattr(module, core)



from django.db.models import Q
from is_core.auth.main import PermissionsMixin

from fperms import get_perm_model

from fperms_iscore import enums
from fperms_iscore.utils import get_iscore_class_str


Perm = get_perm_model()


class PermIsCoreMixin(PermissionsMixin):

    @classmethod
    def _get_perm(cls, codename, obj):
        perms = Perm.objects.filter(
            Q(
                type=enums.PERM_TYPE_CORE,
                codename=codename,
                core=get_iscore_class_str(cls),
            ),
            Q(
                Q(object_id=obj.pk if obj else None) |
                Q(object_id=None)
            )
        )
        try:
            return perms.get()
        except Perm.DoesNotExist:
            return None

    def has_read_permission(self, request, obj=None):
        return request.user.is_superuser or request.user.perms.has_perm(self._get_perm('read', obj=obj))

    def has_create_permission(self, request, obj=None):
        return request.user.is_superuser or request.user.perms.has_perm(self._get_perm('create', obj=obj))

    def has_update_permission(self, request, obj=None):
        return request.user.is_superuser or request.user.perms.has_perm(self._get_perm('update', obj=obj))

    def has_delete_permission(self, request, obj=None):
        return request.user.is_superuser or request.user.perms.has_perm(self._get_perm('delete', obj=obj))



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-04-13 15:20
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
        ('auth', '0008_alter_user_username_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='IsCorePerm',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(choices=[('generic', 'generic'), ('model', 'model'), ('object', 'object'), ('field', 'field')], default='generic', max_length=10)),
                ('codename', models.CharField(max_length=100, verbose_name='codename')),
                ('name', models.CharField(blank=True, max_length=255, null=True, verbose_name='name')),
                ('object_id', models.SmallIntegerField(blank=True, null=True, verbose_name='object pk')),
                ('field_name', models.CharField(blank=True, max_length=100, null=True, verbose_name='field name')),
                ('core', models.CharField(max_length=100, verbose_name='core')),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType', verbose_name='content type')),
                ('groups', models.ManyToManyField(blank=True, related_name='perms', to='auth.Group')),
                ('users', models.ManyToManyField(blank=True, related_name='perms', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='iscoreperm',
            unique_together=set([('type', 'codename', 'content_type', 'object_id', 'field_name', 'core')]),
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _

from fperms.base import BasePerm
from fperms.managers import RelatedPermManager

from fperms_iscore import enums
from fperms_iscore.utils import get_iscore_class
from fperms_iscore.exceptions import IsCoreDoesNotExist


class IsCorePerm(BasePerm):

    PERM_TYPE_CHOICES = enums.PERM_TYPE_CHOICES
    PERM_CODENAMES = enums.PERM_CODENAMES

    core = models.CharField(
        _('core'),
        max_length=100,
    )

    related_manager = RelatedPermManager()

    class Meta:
        base_manager_name = 'related_manager'
        unique_together = (
            ('type', 'codename', 'content_type', 'object_id', 'field_name', 'core'),
        )

    @property
    def _core_perm_name(self):
        return _('core %s') % (
            self.core,
        )

    @property
    def _core_perm_str_args(self):
        return [
            self.type,
            self.core,
            self.codename,
        ]

    @classmethod
    def get_perm_kwargs(cls, perm, obj=None):
        perm_type, perm_arg_string = perm.split('.', 1)

        if perm_type == enums.PERM_TYPE_CORE:
            core, codename = perm_arg_string.rsplit('.', 1)

            try:
                get_iscore_class(core)
            except AttributeError:
                raise IsCoreDoesNotExist(_('provided IsCore does not exist'))

            return dict(
                type=perm_type,
                codename=codename,
                core=core
            )

        return super().get_perm_kwargs(perm=perm, obj=obj)



from django.utils.translation import ugettext_lazy as _

from fperms.enums import *

PERM_TYPE_CORE = 'core'

IS_CORE_PERM_TYPE_CHOICES = (
    (PERM_TYPE_CORE, _('core')),
)

PERM_TYPE_CHOICES = settings.PERM_TYPE_CHOICES + IS_CORE_PERM_TYPE_CHOICES

PERM_CODENAME_CREATE = 'create'
PERM_CODENAME_READ = 'read'

PERM_CODENAMES.update({
    PERM_CODENAME_CREATE: _('create'),
    PERM_CODENAME_READ: _('read'),
})
